# -*- coding: utf-8 -*-
import base64
from typing import Union
from pathlib import Path

from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric import ed25519

from casperlabs_client.consts import ED25519_KEY_ALGORITHM
from casperlabs_client.io import read_binary_file
from .key_holder import KeyHolder


class ED25519Key(KeyHolder):
    """ Class for loading, generating and handling public/private key pairs using ed25519 algorithm """

    def __init__(
        self,
        private_key_pem: bytes = None,
        private_key=None,
        public_key_pem: bytes = None,
        public_key=None,
    ):
        super().__init__(
            private_key_pem,
            private_key,
            public_key_pem,
            public_key,
            ED25519_KEY_ALGORITHM,
        )

    @staticmethod
    def _parse_pem_data_line(pem_file_data: bytes) -> bytes:
        """
        :param pem_file_data: data from pem file as bytes
        :return: raw bytes from the data line(s)
        """
        for line in pem_file_data.split(b"\n"):
            if not line.startswith(b"-----"):
                # ed25519 DER encoding is never longer than one line of base64.
                return line

    @staticmethod
    def _parse_pem_data(pem_file_data: bytes):
        raw_data = ED25519Key._parse_pem_data_line(pem_file_data)
        data = base64.b64decode(raw_data)
        # PKCS8 encoding is longer than 32, but has public and private keys as last 32 bytes.
        # This also allows for a PEM format of ONLY the key (% 32 = 0). I don't know if we should allow this.
        if len(data) % 32 == 0:
            return data[:32]
        else:
            return data[-32:]

    def _private_key_pem_from_private_key(self) -> bytes:
        private_key = ed25519.Ed25519PrivateKey.from_private_bytes(self.private_key)
        return private_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.PKCS8,
            encryption_algorithm=serialization.NoEncryption(),
        )

    def _private_key_from_private_key_pem(self) -> bytes:
        return self._parse_pem_data(self.private_key_pem)

    def _public_key_pem_from_public_key(self) -> bytes:
        public_key = ed25519.Ed25519PublicKey.from_public_bytes(self.public_key)
        return public_key.public_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PublicFormat.SubjectPublicKeyInfo,
        )

    def _public_key_from_public_key_pem(self):
        return self._parse_pem_data(self.public_key_pem)

    def _public_key_from_private_key(self):
        private_key = ed25519.Ed25519PrivateKey.from_private_bytes(self.private_key)
        return private_key.public_key().public_bytes(
            encoding=serialization.Encoding.Raw, format=serialization.PublicFormat.Raw
        )

    def sign(self, data: bytes) -> bytes:
        return ed25519.Ed25519PrivateKey.from_private_bytes(self.private_key).sign(data)

    @staticmethod
    def from_private_key_path(private_key_pem_path: Union[str, Path]) -> "KeyHolder":
        """ Returns a ED25519Key object loaded from a private_key_pem file"""
        private_key_pem = read_binary_file(private_key_pem_path)
        return ED25519Key(private_key_pem=private_key_pem)

    @staticmethod
    def from_public_key_path(public_key_pem_path: Union[str, Path]) -> "KeyHolder":
        """ Returns a ED25519Key object loaded from a private_key_pem file"""
        public_key_pem = read_binary_file(public_key_pem_path)
        return ED25519Key(public_key_pem=public_key_pem)

    @staticmethod
    def generate():
        """
        Generates a new key pair and returns as ED25519 object.

        :returns Ed25519Key object
        """
        private_key = ed25519.Ed25519PrivateKey.generate()

        private_pem = private_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.PKCS8,
            encryption_algorithm=serialization.NoEncryption(),
        )

        private_bytes = private_key.private_bytes(
            encoding=serialization.Encoding.Raw,
            format=serialization.PrivateFormat.Raw,
            encryption_algorithm=serialization.NoEncryption(),
        )

        return ED25519Key(private_key_pem=private_pem, private_key=private_bytes)



# -*- coding: utf-8 -*-
from casperlabs_client import consts
from casperlabs_client.consts import SUPPORTED_KEY_ALGORITHMS, ED25519_KEY_ALGORITHM
from casperlabs_client.arg_types import algorithm, directory_for_write, positive_integer

ALGORITHM_OPTION = [
    ("--algorithm",),
    dict(
        required=False,
        type=algorithm,
        choices=SUPPORTED_KEY_ALGORITHMS,
        default=ED25519_KEY_ALGORITHM,
        help=f"Algorithm used for public key generation. Default is {ED25519_KEY_ALGORITHM}.",
    ),
]

FROM_OPTION = [
    ("-f", "--from"),
    dict(
        required=False,
        type=str,
        help="The account hash of the account which is the context of this deployment, base16 encoded.",
    ),
]

DIRECTORY_FOR_WRITE_OPTION = [
    ("directory",),
    dict(type=directory_for_write, help="Output directory. Should already exist."),
]


PAYMENT_OPTIONS = [
    [
        ("-p", "--payment"),
        dict(
            required=False,
            type=str,
            default=None,
            help="Path to the file with payment code",
        ),
    ],
    [
        ("--payment-amount",),
        dict(
            required=False,
            type=int,
            default=None,
            help=(
                "Standard payment amount. Use this with the default payment, or override with --payment-args "
                "if custom payment code is used. By default --payment-amount is set to 10000000"
            ),
        ),
    ],
    [
        ("--payment-hash",),
        dict(
            required=False,
            type=str,
            default=None,
            help="Hash of the stored contract to be called in the payment; base16 encoded",
        ),
    ],
    [
        ("--payment-name",),
        dict(
            required=False,
            type=str,
            default=None,
            help="Name of the stored contract (associated with the executing account) to be called in the payment",
        ),
    ],
    [
        ("--payment-package-hash",),
        dict(
            required=False,
            type=str,
            default=None,
            help="Hash of the stored package to be called in the payment; base16 encoded",
        ),
    ],
    [
        ("--payment-package-name",),
        dict(
            required=False,
            type=str,
            default=None,
            help="Name of the stored package (associated with the executing account) to be called in the payment",
        ),
    ],
    [
        ("--payment-entry-point",),
        dict(
            required=False,
            type=str,
            default=None,
            help="Name of the method that will be used when calling the payment contract.",
        ),
    ],
    [
        ("--payment-version",),
        dict(
            required=False,
            type=positive_integer,
            default=None,
            help="Version of the called payment contract.  Latest will be used by default",
        ),
    ],
    [
        ("--payment-args",),
        dict(
            required=False,
            type=str,
            help="""JSON encoded list of payment args, e.g.: '[{"name": "amount", "value": {"big_int": {"value": "123456", "bit_width": 512}}}]'""",
        ),
    ],
]

SESSION_OPTIONS = [
    [
        ("-s", "--session"),
        dict(
            required=False,
            type=str,
            default=None,
            help="Path to the file with session code",
        ),
    ],
    [
        ("--session-hash",),
        dict(
            required=False,
            type=str,
            default=None,
            help="Hash of the stored contract to be called in the session; base16 encoded",
        ),
    ],
    [
        ("--session-name",),
        dict(
            required=False,
            type=str,
            default=None,
            help="Name of the stored contract (associated with the executing account) to be called in the session",
        ),
    ],
    [
        ("--session-package-hash",),
        dict(
            required=False,
            type=str,
            default=None,
            help="Hash of the stored package to be called in the session; base16 encoded",
        ),
    ],
    [
        ("--session-package-name",),
        dict(
            required=False,
            type=str,
            default=None,
            help="Name of the stored package (associated with the executing account) to be called in the session",
        ),
    ],
    [
        ("--session-entry-point",),
        dict(
            required=False,
            type=str,
            default=None,
            help="Name of the method that will be used when calling the session contract.",
        ),
    ],
    [
        ("--session-version",),
        dict(
            required=False,
            type=positive_integer,
            default=None,
            help="Version of the called session contract. Latest will be used by default.",
        ),
    ],
    [
        ("--session-args",),
        dict(
            required=False,
            type=str,
            help="""JSON encoded list of session args, e.g.: '[{"name": "amount", "value": {"long_value": 123456}}]'""",
        ),
    ],
]

CHAINNAME_OPTION = [
    ("--chain-name",),
    dict(
        required=False,
        type=str,
        help="Name of the chain to optionally restrict the deploy from being accidentally included anywhere else.",
    ),
]

DEPENDENCIES_OPTION = [
    ("--dependencies",),
    dict(
        required=False,
        nargs="+",
        default=None,
        help="List of deploy hashes (base16 encoded) which must be executed before this deploy.",
    ),
]


def private_key_option(required=False):
    return [
        ("--private-key",),
        dict(
            required=required,
            default=None,
            type=str,
            help="Path to the file with account private key.  Assumed to be Ed25519, unless --algorithm is given.",
        ),
    ]


def public_key_option(required=False):
    return [
        ("--public-key",),
        dict(
            required=required,
            default=None,
            type=str,
            help="Path to the file with account public key.  Assumed to be Ed25519, unless --algorithm is given.",
        ),
    ]


TTL_MILLIS_OPTION = [
    ("--ttl-millis",),
    dict(
        required=False,
        type=int,
        help="Time to live. Time (in milliseconds) that the deploy will remain valid for.'",
    ),
]

WAIT_PROCESSED_OPTION = [
    ("-w", "--wait-for-processed"),
    dict(action="store_true", help="Wait for deploy status PROCESSED or DISCARDED"),
]

TIMEOUT_SECONDS_OPTION = [
    ("--timeout-seconds",),
    dict(type=int, default=consts.STATUS_TIMEOUT, help="Timeout in seconds"),
]


DEPLOY_OPTIONS = (
    [
        FROM_OPTION,
        CHAINNAME_OPTION,
        DEPENDENCIES_OPTION,
        TTL_MILLIS_OPTION,
        WAIT_PROCESSED_OPTION,
        TIMEOUT_SECONDS_OPTION,
        ALGORITHM_OPTION,
    ]
    + SESSION_OPTIONS
    + PAYMENT_OPTIONS
)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: diagnostics.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import empty_pb2 as empty__pb2
from . import node_pb2 as node__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='diagnostics.proto',
  package='io.casperlabs.node.api.diagnostics',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x11\x64iagnostics.proto\x12\"io.casperlabs.node.api.diagnostics\x1a\x0b\x65mpty.proto\x1a\nnode.proto\"\xfd\x01\n\x0fNodeCoreMetrics\x12\x19\n\x11pingReceiverCount\x18\x01 \x01(\x03\x12\x1b\n\x13lookupReceiverCount\x18\x02 \x01(\x03\x12\x1f\n\x17\x64isconnectReceiverCount\x18\x03 \x01(\x03\x12\x10\n\x08\x63onnects\x18\x04 \x01(\x03\x12+\n#p2pEncryptionHandshakeReceiverCount\x18\x05 \x01(\x03\x12)\n!p2pProtocolHandshakeReceiverCount\x18\x06 \x01(\x03\x12\r\n\x05peers\x18\x07 \x01(\x03\x12\x0c\n\x04\x66rom\x18\x08 \x01(\x03\x12\n\n\x02to\x18\t \x01(\x03\":\n\x05Peers\x12\x31\n\x05peers\x18\x01 \x03(\x0b\x32\".io.casperlabs.comm.discovery.Node2]\n\x0b\x44iagnostics\x12N\n\tListPeers\x12\x16.google.protobuf.Empty\x1a).io.casperlabs.node.api.diagnostics.Peersb\x06proto3'
  ,
  dependencies=[empty__pb2.DESCRIPTOR,node__pb2.DESCRIPTOR,])




_NODECOREMETRICS = _descriptor.Descriptor(
  name='NodeCoreMetrics',
  full_name='io.casperlabs.node.api.diagnostics.NodeCoreMetrics',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='pingReceiverCount', full_name='io.casperlabs.node.api.diagnostics.NodeCoreMetrics.pingReceiverCount', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='lookupReceiverCount', full_name='io.casperlabs.node.api.diagnostics.NodeCoreMetrics.lookupReceiverCount', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='disconnectReceiverCount', full_name='io.casperlabs.node.api.diagnostics.NodeCoreMetrics.disconnectReceiverCount', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='connects', full_name='io.casperlabs.node.api.diagnostics.NodeCoreMetrics.connects', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='p2pEncryptionHandshakeReceiverCount', full_name='io.casperlabs.node.api.diagnostics.NodeCoreMetrics.p2pEncryptionHandshakeReceiverCount', index=4,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='p2pProtocolHandshakeReceiverCount', full_name='io.casperlabs.node.api.diagnostics.NodeCoreMetrics.p2pProtocolHandshakeReceiverCount', index=5,
      number=6, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='peers', full_name='io.casperlabs.node.api.diagnostics.NodeCoreMetrics.peers', index=6,
      number=7, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='from', full_name='io.casperlabs.node.api.diagnostics.NodeCoreMetrics.from', index=7,
      number=8, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='to', full_name='io.casperlabs.node.api.diagnostics.NodeCoreMetrics.to', index=8,
      number=9, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=83,
  serialized_end=336,
)


_PEERS = _descriptor.Descriptor(
  name='Peers',
  full_name='io.casperlabs.node.api.diagnostics.Peers',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='peers', full_name='io.casperlabs.node.api.diagnostics.Peers.peers', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=338,
  serialized_end=396,
)

_PEERS.fields_by_name['peers'].message_type = node__pb2._NODE
DESCRIPTOR.message_types_by_name['NodeCoreMetrics'] = _NODECOREMETRICS
DESCRIPTOR.message_types_by_name['Peers'] = _PEERS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

NodeCoreMetrics = _reflection.GeneratedProtocolMessageType('NodeCoreMetrics', (_message.Message,), {
  'DESCRIPTOR' : _NODECOREMETRICS,
  '__module__' : 'diagnostics_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.node.api.diagnostics.NodeCoreMetrics)
  })
_sym_db.RegisterMessage(NodeCoreMetrics)

Peers = _reflection.GeneratedProtocolMessageType('Peers', (_message.Message,), {
  'DESCRIPTOR' : _PEERS,
  '__module__' : 'diagnostics_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.node.api.diagnostics.Peers)
  })
_sym_db.RegisterMessage(Peers)



_DIAGNOSTICS = _descriptor.ServiceDescriptor(
  name='Diagnostics',
  full_name='io.casperlabs.node.api.diagnostics.Diagnostics',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=398,
  serialized_end=491,
  methods=[
  _descriptor.MethodDescriptor(
    name='ListPeers',
    full_name='io.casperlabs.node.api.diagnostics.Diagnostics.ListPeers',
    index=0,
    containing_service=None,
    input_type=empty__pb2._EMPTY,
    output_type=_PEERS,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_DIAGNOSTICS)

DESCRIPTOR.services_by_name['Diagnostics'] = _DIAGNOSTICS

# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
from pathlib import Path

from casperlabs_client import CasperLabsClient
from casperlabs_client.commands.common_options import (
    ALGORITHM_OPTION,
    DIRECTORY_FOR_WRITE_OPTION,
)
from casperlabs_client.decorators import guarded_command

NAME: str = "keygen"
HELP: str = """Generates account keys into existing directory

Usage: casperlabs-client keygen <existing output directory>
Command will override existing files!
Generated files:
   account-id-hex       # Hash of public key for use on the system as hex text
   account-private.pem  # private key based on given algorithm, default ed25519
   account-public.pem   # public key based on given algorithm, default ed25519"""
OPTIONS = (DIRECTORY_FOR_WRITE_OPTION, ALGORITHM_OPTION)


@guarded_command
def method(casperlabs_client: CasperLabsClient, args):
    directory = Path(args.get("directory")).resolve()
    algorithm = args.get("algorithm")
    casperlabs_client.keygen(directory, algorithm)
    print(f"Keys successfully created in directory: {str(directory.absolute())}")



# Generated by the Protocol Buffers compiler. DO NOT EDIT!
# source: casper.proto
# plugin: grpclib.plugin.main
import abc
import typing

import grpclib.const
import grpclib.client
if typing.TYPE_CHECKING:
    import grpclib.server

import annotations_pb2
import empty_pb2
import consensus_pb2
import info_pb2
import state_pb2
import casper_pb2


class CasperServiceBase(abc.ABC):

    @abc.abstractmethod
    async def Deploy(self, stream: 'grpclib.server.Stream[casper_pb2.DeployRequest, empty_pb2.Empty]') -> None:
        pass

    @abc.abstractmethod
    async def GetBlockInfo(self, stream: 'grpclib.server.Stream[casper_pb2.GetBlockInfoRequest, info_pb2.BlockInfo]') -> None:
        pass

    @abc.abstractmethod
    async def StreamBlockInfos(self, stream: 'grpclib.server.Stream[casper_pb2.StreamBlockInfosRequest, info_pb2.BlockInfo]') -> None:
        pass

    @abc.abstractmethod
    async def GetDeployInfo(self, stream: 'grpclib.server.Stream[casper_pb2.GetDeployInfoRequest, info_pb2.DeployInfo]') -> None:
        pass

    @abc.abstractmethod
    async def StreamBlockDeploys(self, stream: 'grpclib.server.Stream[casper_pb2.StreamBlockDeploysRequest, consensus_pb2.Block.ProcessedDeploy]') -> None:
        pass

    @abc.abstractmethod
    async def StreamEvents(self, stream: 'grpclib.server.Stream[casper_pb2.StreamEventsRequest, info_pb2.Event]') -> None:
        pass

    @abc.abstractmethod
    async def GetBlockState(self, stream: 'grpclib.server.Stream[casper_pb2.GetBlockStateRequest, state_pb2.StoredValueInstance]') -> None:
        pass

    @abc.abstractmethod
    async def BatchGetBlockState(self, stream: 'grpclib.server.Stream[casper_pb2.BatchGetBlockStateRequest, casper_pb2.BatchGetBlockStateResponse]') -> None:
        pass

    @abc.abstractmethod
    async def ListDeployInfos(self, stream: 'grpclib.server.Stream[casper_pb2.ListDeployInfosRequest, casper_pb2.ListDeployInfosResponse]') -> None:
        pass

    @abc.abstractmethod
    async def GetLastFinalizedBlockInfo(self, stream: 'grpclib.server.Stream[casper_pb2.GetLastFinalizedBlockInfoRequest, info_pb2.BlockInfo]') -> None:
        pass

    def __mapping__(self) -> typing.Dict[str, grpclib.const.Handler]:
        return {
            '/io.casperlabs.node.api.casper.CasperService/Deploy': grpclib.const.Handler(
                self.Deploy,
                grpclib.const.Cardinality.UNARY_UNARY,
                casper_pb2.DeployRequest,
                empty_pb2.Empty,
            ),
            '/io.casperlabs.node.api.casper.CasperService/GetBlockInfo': grpclib.const.Handler(
                self.GetBlockInfo,
                grpclib.const.Cardinality.UNARY_UNARY,
                casper_pb2.GetBlockInfoRequest,
                info_pb2.BlockInfo,
            ),
            '/io.casperlabs.node.api.casper.CasperService/StreamBlockInfos': grpclib.const.Handler(
                self.StreamBlockInfos,
                grpclib.const.Cardinality.UNARY_STREAM,
                casper_pb2.StreamBlockInfosRequest,
                info_pb2.BlockInfo,
            ),
            '/io.casperlabs.node.api.casper.CasperService/GetDeployInfo': grpclib.const.Handler(
                self.GetDeployInfo,
                grpclib.const.Cardinality.UNARY_UNARY,
                casper_pb2.GetDeployInfoRequest,
                info_pb2.DeployInfo,
            ),
            '/io.casperlabs.node.api.casper.CasperService/StreamBlockDeploys': grpclib.const.Handler(
                self.StreamBlockDeploys,
                grpclib.const.Cardinality.UNARY_STREAM,
                casper_pb2.StreamBlockDeploysRequest,
                consensus_pb2.Block.ProcessedDeploy,
            ),
            '/io.casperlabs.node.api.casper.CasperService/StreamEvents': grpclib.const.Handler(
                self.StreamEvents,
                grpclib.const.Cardinality.UNARY_STREAM,
                casper_pb2.StreamEventsRequest,
                info_pb2.Event,
            ),
            '/io.casperlabs.node.api.casper.CasperService/GetBlockState': grpclib.const.Handler(
                self.GetBlockState,
                grpclib.const.Cardinality.UNARY_UNARY,
                casper_pb2.GetBlockStateRequest,
                state_pb2.StoredValueInstance,
            ),
            '/io.casperlabs.node.api.casper.CasperService/BatchGetBlockState': grpclib.const.Handler(
                self.BatchGetBlockState,
                grpclib.const.Cardinality.UNARY_UNARY,
                casper_pb2.BatchGetBlockStateRequest,
                casper_pb2.BatchGetBlockStateResponse,
            ),
            '/io.casperlabs.node.api.casper.CasperService/ListDeployInfos': grpclib.const.Handler(
                self.ListDeployInfos,
                grpclib.const.Cardinality.UNARY_UNARY,
                casper_pb2.ListDeployInfosRequest,
                casper_pb2.ListDeployInfosResponse,
            ),
            '/io.casperlabs.node.api.casper.CasperService/GetLastFinalizedBlockInfo': grpclib.const.Handler(
                self.GetLastFinalizedBlockInfo,
                grpclib.const.Cardinality.UNARY_UNARY,
                casper_pb2.GetLastFinalizedBlockInfoRequest,
                info_pb2.BlockInfo,
            ),
        }


class CasperServiceStub:

    def __init__(self, channel: grpclib.client.Channel) -> None:
        self.Deploy = grpclib.client.UnaryUnaryMethod(
            channel,
            '/io.casperlabs.node.api.casper.CasperService/Deploy',
            casper_pb2.DeployRequest,
            empty_pb2.Empty,
        )
        self.GetBlockInfo = grpclib.client.UnaryUnaryMethod(
            channel,
            '/io.casperlabs.node.api.casper.CasperService/GetBlockInfo',
            casper_pb2.GetBlockInfoRequest,
            info_pb2.BlockInfo,
        )
        self.StreamBlockInfos = grpclib.client.UnaryStreamMethod(
            channel,
            '/io.casperlabs.node.api.casper.CasperService/StreamBlockInfos',
            casper_pb2.StreamBlockInfosRequest,
            info_pb2.BlockInfo,
        )
        self.GetDeployInfo = grpclib.client.UnaryUnaryMethod(
            channel,
            '/io.casperlabs.node.api.casper.CasperService/GetDeployInfo',
            casper_pb2.GetDeployInfoRequest,
            info_pb2.DeployInfo,
        )
        self.StreamBlockDeploys = grpclib.client.UnaryStreamMethod(
            channel,
            '/io.casperlabs.node.api.casper.CasperService/StreamBlockDeploys',
            casper_pb2.StreamBlockDeploysRequest,
            consensus_pb2.Block.ProcessedDeploy,
        )
        self.StreamEvents = grpclib.client.UnaryStreamMethod(
            channel,
            '/io.casperlabs.node.api.casper.CasperService/StreamEvents',
            casper_pb2.StreamEventsRequest,
            info_pb2.Event,
        )
        self.GetBlockState = grpclib.client.UnaryUnaryMethod(
            channel,
            '/io.casperlabs.node.api.casper.CasperService/GetBlockState',
            casper_pb2.GetBlockStateRequest,
            state_pb2.StoredValueInstance,
        )
        self.BatchGetBlockState = grpclib.client.UnaryUnaryMethod(
            channel,
            '/io.casperlabs.node.api.casper.CasperService/BatchGetBlockState',
            casper_pb2.BatchGetBlockStateRequest,
            casper_pb2.BatchGetBlockStateResponse,
        )
        self.ListDeployInfos = grpclib.client.UnaryUnaryMethod(
            channel,
            '/io.casperlabs.node.api.casper.CasperService/ListDeployInfos',
            casper_pb2.ListDeployInfosRequest,
            casper_pb2.ListDeployInfosResponse,
        )
        self.GetLastFinalizedBlockInfo = grpclib.client.UnaryUnaryMethod(
            channel,
            '/io.casperlabs.node.api.casper.CasperService/GetLastFinalizedBlockInfo',
            casper_pb2.GetLastFinalizedBlockInfoRequest,
            info_pb2.BlockInfo,
        )



# -*- coding: utf-8 -*-
"""
Cryptography related code used in the CasperLabs client.
"""

import datetime
import ssl

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric import ec
from cryptography import x509
from cryptography.hazmat.primitives import hashes
from cryptography.x509.oid import NameOID
from Crypto.Hash import keccak
from pyblake2 import blake2b


def extract_common_name(certificate_file: str) -> str:
    cert_dict = ssl._ssl._test_decode_cert(certificate_file)
    return [t[0][1] for t in cert_dict["subject"] if t[0][0] == "commonName"][0]


def generate_secp256r1_key_pair():
    curve = ec.SECP256R1()
    private_key = ec.generate_private_key(curve, default_backend())
    public_key = private_key.public_key()
    return private_key, public_key


def node_public_address(public_key):
    numbers = public_key.public_numbers()
    x, y = numbers.x, numbers.y

    def int_to_32_bytes(x):
        return x.to_bytes(x.bit_length(), byteorder="little")[0:32]

    a = int_to_32_bytes(x) + int_to_32_bytes(y)

    keccak_hash = keccak.new(digest_bits=256)
    keccak_hash.update(a)
    r = keccak_hash.hexdigest()
    return r[12 * 2 :]


def generate_node_certificates(private_key, public_key):
    today = datetime.datetime.today()
    one_day = datetime.timedelta(1, 0, 0)
    address = node_public_address(
        public_key
    )  # .map(Base16.to_protobuf).getOrElse("local")
    owner = f"CN={address}"

    builder = x509.CertificateBuilder()
    builder = builder.not_valid_before(today)

    # TODO: Where's documentation of the decision to make keys valid for 1 year only?
    builder = builder.not_valid_after(today + 365 * one_day)
    builder = builder.subject_name(
        x509.Name([x509.NameAttribute(NameOID.COMMON_NAME, owner)])
    )
    builder = builder.issuer_name(
        x509.Name([x509.NameAttribute(NameOID.COMMON_NAME, owner)])
    )
    builder = builder.public_key(public_key)
    builder = builder.serial_number(x509.random_serial_number())
    certificate = builder.sign(
        private_key=private_key, algorithm=hashes.SHA256(), backend=default_backend()
    )

    cert_pem = certificate.public_bytes(encoding=serialization.Encoding.PEM)
    key_pem = private_key.private_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PrivateFormat.PKCS8,
        encryption_algorithm=serialization.NoEncryption(),
    )
    return cert_pem, key_pem


def blake2b_hash(data: bytes) -> bytes:
    h = blake2b(digest_size=32)
    h.update(data)
    return h.digest()



# -*- coding: utf-8 -*-
from dataclasses import dataclass
from typing import Dict, Union

from . import consensus_pb2 as consensus, consts, key_holders
import time

from casperlabs_client import crypto
from .contract import PaymentCode, SessionCode
from .key_holders import ED25519Key, SECP256K1Key


def sign_deploy(deploy, key_holder):
    signature_bytes = key_holder.sign(deploy.deploy_hash)
    signature = consensus.Signature(
        sig_algorithm=key_holder.algorithm.lower(), sig=signature_bytes
    )
    deploy.approvals.extend(
        [
            consensus.Approval(
                approver_public_key=key_holder.public_key, signature=signature
            )
        ]
    )
    return deploy


@dataclass
class DeployData:
    from_addr: bytes
    payment_code: PaymentCode
    session_code: SessionCode
    ttl_millis: int = 0
    dependencies: list = None
    chain_name: str = None
    key_holder: Union[ED25519Key, SECP256K1Key] = None

    @staticmethod
    def from_args(args: Dict) -> "DeployData":
        """
        Build Deploy data class from command line arguments, this includes testing for combinations of args
        that are allowed or required.
        """

        # `from` isn't good for dict creation, but used from CLI, so handle both `from` and `from_addr`
        from_addr = args.get("from", args.get("from_addr"))
        if from_addr and not isinstance(from_addr, bytes):
            from_addr = bytes.fromhex(from_addr)

        payment_code = PaymentCode.from_args(args)
        session_code = SessionCode.from_args(args)

        ttl_millis = args.get("ttl_millis")
        dependencies_hex = args.get("dependencies") or []
        dependencies = [bytes.fromhex(d) for d in dependencies_hex]
        chain_name = args.get("chain_name", "")
        algorithm = args.get("algorithm")
        private_key_pem_path = args.get("private_key")

        private_key_hex = args.get("private_key_hex")
        private_key = bytes.fromhex(private_key_hex) if private_key_hex else None
        public_key_hex = args.get("public_key_hex")
        public_key = bytes.fromhex(public_key_hex) if public_key_hex else None

        public_key_pem_path = args.get("public_key")
        try:
            key_holder = key_holders.key_holder_object(
                algorithm=algorithm,
                private_key_pem_path=private_key_pem_path,
                public_key_pem_path=public_key_pem_path,
                private_key=private_key,
                public_key=public_key,
            )
        except ValueError:
            key_holder = None

        if not from_addr:
            if not key_holder:
                raise ValueError(
                    "Must provide `from` or a key to calculate account hash."
                )
            from_addr = key_holder.account_hash

        deploy = DeployData(
            from_addr=from_addr,
            payment_code=payment_code,
            session_code=session_code,
            ttl_millis=ttl_millis,
            dependencies=dependencies,
            chain_name=chain_name,
            key_holder=key_holder,
        )

        if len(deploy.from_addr) != consts.ACCOUNT_HASH_LENGTH:
            raise Exception(
                "--from must be 32 bytes encoded as 64 characters long hexadecimal"
            )

        return deploy

    def make_protobuf(self) -> consensus.Deploy:
        """
        Create a protobuf deploy object. See deploy for description of parameters.
        """
        if len(self.from_addr) != 32:
            raise Exception("from_addr must be 32 bytes")

        self.payment_code.validate()
        self.session_code.validate()

        body = consensus.Deploy.Body(
            session=self.session_code.to_protobuf(),
            payment=self.payment_code.to_protobuf(),
        )
        body_hash = crypto.blake2b_hash((body.SerializeToString()))

        header = consensus.Deploy.Header(
            account_public_key_hash=self.from_addr,
            timestamp=int(1000 * time.time()),
            body_hash=body_hash,
            ttl_millis=self.ttl_millis,
            dependencies=self.dependencies,
            chain_name=self.chain_name,
        )

        deploy_hash = crypto.blake2b_hash(header.SerializeToString())

        return consensus.Deploy(deploy_hash=deploy_hash, header=header, body=body)



# -*- coding: utf-8 -*-
from typing import Dict

from casperlabs_client import consts, reformat, CasperLabsClient
from casperlabs_client.commands.common_options import private_key_option, DEPLOY_OPTIONS
from casperlabs_client.decorators import guarded_command

NAME: str = "deploy"
HELP: str = (
    "Deploy a smart contract source file to Casper on an existing running node. "
    "The deploy will be packaged and sent as a block to the network depending "
    "on the configuration of the Casper instance."
)

OPTIONS = DEPLOY_OPTIONS + [private_key_option(required=True)]


@guarded_command
def method(casperlabs_client: CasperLabsClient, args: Dict):
    deploy_hash = casperlabs_client.deploy(
        from_addr=args.get("from"),
        payment=args.get("payment"),
        session=args.get("session"),
        private_key=args.get("private_key"),
        session_args=args.get("session_args"),
        payment_args=args.get("payment_args"),
        payment_amount=args.get("payment_amount"),
        payment_hash=args.get("payment_hash"),
        payment_name=args.get("payment_name"),
        payment_package_hash=args.get("payment_package_hash"),
        payment_package_name=args.get("payment_package_name"),
        session_hash=args.get("session_hash"),
        session_name=args.get("session_name"),
        session_package_hash=args.get("session_package_hash"),
        session_package_name=args.get("session_package_name"),
        ttl_millis=args.get("ttl_millis"),
        dependencies=args.get("dependencies"),
        chain_name=args.get("chain_name"),
        algorithm=args.get("algorithm"),
    )
    print(f"Success! Deploy {deploy_hash} deployed")
    if args.get("wait_for_processed", False):
        deploy_info = casperlabs_client.show_deploy(
            deploy_hash,
            full_view=False,
            wait_for_processed=True,
            timeout_seconds=args.get("timeout_seconds", consts.STATUS_TIMEOUT),
        )
        print(reformat.hexify(deploy_info))



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: gossiping.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import empty_pb2 as empty__pb2
from . import consensus_pb2 as consensus__pb2
from . import node_pb2 as node__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='gossiping.proto',
  package='io.casperlabs.comm.gossiping',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0fgossiping.proto\x12\x1cio.casperlabs.comm.gossiping\x1a\x0b\x65mpty.proto\x1a\x0f\x63onsensus.proto\x1a\nnode.proto\"\\\n\x10NewBlocksRequest\x12\x32\n\x06sender\x18\x01 \x01(\x0b\x32\".io.casperlabs.comm.discovery.Node\x12\x14\n\x0c\x62lock_hashes\x18\x02 \x03(\x0c\"#\n\x11NewBlocksResponse\x12\x0e\n\x06is_new\x18\x01 \x01(\x08\"^\n\x11NewDeploysRequest\x12\x32\n\x06sender\x18\x01 \x01(\x0b\x32\".io.casperlabs.comm.discovery.Node\x12\x15\n\rdeploy_hashes\x18\x02 \x03(\x0c\"$\n\x12NewDeploysResponse\x12\x0e\n\x06is_new\x18\x01 \x01(\x08\"3\n\x1bStreamBlockSummariesRequest\x12\x14\n\x0c\x62lock_hashes\x18\x01 \x03(\x0c\"5\n\x1cStreamDeploySummariesRequest\x12\x15\n\rdeploy_hashes\x18\x01 \x03(\x0c\"q\n#StreamAncestorBlockSummariesRequest\x12\x1b\n\x13target_block_hashes\x18\x01 \x03(\x0c\x12\x1a\n\x12known_block_hashes\x18\x02 \x03(\x0c\x12\x11\n\tmax_depth\x18\x03 \x01(\r\"\x1d\n\x1bStreamLatestMessagesRequest\"\xac\x01\n\x16GetBlockChunkedRequest\x12\x12\n\nblock_hash\x18\x01 \x01(\x0c\x12\x12\n\nchunk_size\x18\x02 \x01(\r\x12\'\n\x1f\x61\x63\x63\x65pted_compression_algorithms\x18\x03 \x03(\t\x12\x1d\n\x15\x65xclude_deploy_bodies\x18\x04 \x01(\x08\x12\"\n\x1aonly_include_deploy_hashes\x18\x05 \x01(\x08\"q\n\x1bStreamDeploysChunkedRequest\x12\x15\n\rdeploy_hashes\x18\x01 \x03(\x0c\x12\x12\n\nchunk_size\x18\x02 \x01(\r\x12\'\n\x1f\x61\x63\x63\x65pted_compression_algorithms\x18\x03 \x03(\t\"\x1c\n\x1aGetGenesisCandidateRequest\"d\n\x12\x41\x64\x64\x41pprovalRequest\x12\x12\n\nblock_hash\x18\x01 \x01(\x0c\x12:\n\x08\x61pproval\x18\x02 \x01(\x0b\x32(.io.casperlabs.casper.consensus.Approval\"\xc2\x01\n\x05\x43hunk\x12<\n\x06header\x18\x01 \x01(\x0b\x32*.io.casperlabs.comm.gossiping.Chunk.HeaderH\x00\x12\x0e\n\x04\x64\x61ta\x18\x02 \x01(\x0cH\x00\x1a`\n\x06Header\x12\x1d\n\x15\x63ompression_algorithm\x18\x01 \x01(\t\x12\x16\n\x0e\x63ontent_length\x18\x02 \x01(\r\x12\x1f\n\x17original_content_length\x18\x03 \x01(\rB\t\n\x07\x63ontent\"K\n#StreamDagSliceBlockSummariesRequest\x12\x12\n\nstart_rank\x18\x01 \x01(\x04\x12\x10\n\x08\x65nd_rank\x18\x02 \x01(\x04\x32\xf3\n\n\rGossipService\x12l\n\tNewBlocks\x12..io.casperlabs.comm.gossiping.NewBlocksRequest\x1a/.io.casperlabs.comm.gossiping.NewBlocksResponse\x12o\n\nNewDeploys\x12/.io.casperlabs.comm.gossiping.NewDeploysRequest\x1a\x30.io.casperlabs.comm.gossiping.NewDeploysResponse\x12\x91\x01\n\x1cStreamAncestorBlockSummaries\x12\x41.io.casperlabs.comm.gossiping.StreamAncestorBlockSummariesRequest\x1a,.io.casperlabs.casper.consensus.BlockSummary0\x01\x12\x88\x01\n\x14StreamLatestMessages\x12\x39.io.casperlabs.comm.gossiping.StreamLatestMessagesRequest\x1a\x33.io.casperlabs.casper.consensus.Block.Justification0\x01\x12\x81\x01\n\x14StreamBlockSummaries\x12\x39.io.casperlabs.comm.gossiping.StreamBlockSummariesRequest\x1a,.io.casperlabs.casper.consensus.BlockSummary0\x01\x12\x84\x01\n\x15StreamDeploySummaries\x12:.io.casperlabs.comm.gossiping.StreamDeploySummariesRequest\x1a-.io.casperlabs.casper.consensus.DeploySummary0\x01\x12n\n\x0fGetBlockChunked\x12\x34.io.casperlabs.comm.gossiping.GetBlockChunkedRequest\x1a#.io.casperlabs.comm.gossiping.Chunk0\x01\x12x\n\x14StreamDeploysChunked\x12\x39.io.casperlabs.comm.gossiping.StreamDeploysChunkedRequest\x1a#.io.casperlabs.comm.gossiping.Chunk0\x01\x12\x81\x01\n\x13GetGenesisCandidate\x12\x38.io.casperlabs.comm.gossiping.GetGenesisCandidateRequest\x1a\x30.io.casperlabs.casper.consensus.GenesisCandidate\x12W\n\x0b\x41\x64\x64\x41pproval\x12\x30.io.casperlabs.comm.gossiping.AddApprovalRequest\x1a\x16.google.protobuf.Empty\x12\x91\x01\n\x1cStreamDagSliceBlockSummaries\x12\x41.io.casperlabs.comm.gossiping.StreamDagSliceBlockSummariesRequest\x1a,.io.casperlabs.casper.consensus.BlockSummary0\x01\x62\x06proto3'
  ,
  dependencies=[empty__pb2.DESCRIPTOR,consensus__pb2.DESCRIPTOR,node__pb2.DESCRIPTOR,])




_NEWBLOCKSREQUEST = _descriptor.Descriptor(
  name='NewBlocksRequest',
  full_name='io.casperlabs.comm.gossiping.NewBlocksRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='sender', full_name='io.casperlabs.comm.gossiping.NewBlocksRequest.sender', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='block_hashes', full_name='io.casperlabs.comm.gossiping.NewBlocksRequest.block_hashes', index=1,
      number=2, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=91,
  serialized_end=183,
)


_NEWBLOCKSRESPONSE = _descriptor.Descriptor(
  name='NewBlocksResponse',
  full_name='io.casperlabs.comm.gossiping.NewBlocksResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='is_new', full_name='io.casperlabs.comm.gossiping.NewBlocksResponse.is_new', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=185,
  serialized_end=220,
)


_NEWDEPLOYSREQUEST = _descriptor.Descriptor(
  name='NewDeploysRequest',
  full_name='io.casperlabs.comm.gossiping.NewDeploysRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='sender', full_name='io.casperlabs.comm.gossiping.NewDeploysRequest.sender', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='deploy_hashes', full_name='io.casperlabs.comm.gossiping.NewDeploysRequest.deploy_hashes', index=1,
      number=2, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=222,
  serialized_end=316,
)


_NEWDEPLOYSRESPONSE = _descriptor.Descriptor(
  name='NewDeploysResponse',
  full_name='io.casperlabs.comm.gossiping.NewDeploysResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='is_new', full_name='io.casperlabs.comm.gossiping.NewDeploysResponse.is_new', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=318,
  serialized_end=354,
)


_STREAMBLOCKSUMMARIESREQUEST = _descriptor.Descriptor(
  name='StreamBlockSummariesRequest',
  full_name='io.casperlabs.comm.gossiping.StreamBlockSummariesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='block_hashes', full_name='io.casperlabs.comm.gossiping.StreamBlockSummariesRequest.block_hashes', index=0,
      number=1, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=356,
  serialized_end=407,
)


_STREAMDEPLOYSUMMARIESREQUEST = _descriptor.Descriptor(
  name='StreamDeploySummariesRequest',
  full_name='io.casperlabs.comm.gossiping.StreamDeploySummariesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='deploy_hashes', full_name='io.casperlabs.comm.gossiping.StreamDeploySummariesRequest.deploy_hashes', index=0,
      number=1, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=409,
  serialized_end=462,
)


_STREAMANCESTORBLOCKSUMMARIESREQUEST = _descriptor.Descriptor(
  name='StreamAncestorBlockSummariesRequest',
  full_name='io.casperlabs.comm.gossiping.StreamAncestorBlockSummariesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='target_block_hashes', full_name='io.casperlabs.comm.gossiping.StreamAncestorBlockSummariesRequest.target_block_hashes', index=0,
      number=1, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='known_block_hashes', full_name='io.casperlabs.comm.gossiping.StreamAncestorBlockSummariesRequest.known_block_hashes', index=1,
      number=2, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='max_depth', full_name='io.casperlabs.comm.gossiping.StreamAncestorBlockSummariesRequest.max_depth', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=464,
  serialized_end=577,
)


_STREAMLATESTMESSAGESREQUEST = _descriptor.Descriptor(
  name='StreamLatestMessagesRequest',
  full_name='io.casperlabs.comm.gossiping.StreamLatestMessagesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=579,
  serialized_end=608,
)


_GETBLOCKCHUNKEDREQUEST = _descriptor.Descriptor(
  name='GetBlockChunkedRequest',
  full_name='io.casperlabs.comm.gossiping.GetBlockChunkedRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='block_hash', full_name='io.casperlabs.comm.gossiping.GetBlockChunkedRequest.block_hash', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='chunk_size', full_name='io.casperlabs.comm.gossiping.GetBlockChunkedRequest.chunk_size', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='accepted_compression_algorithms', full_name='io.casperlabs.comm.gossiping.GetBlockChunkedRequest.accepted_compression_algorithms', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='exclude_deploy_bodies', full_name='io.casperlabs.comm.gossiping.GetBlockChunkedRequest.exclude_deploy_bodies', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='only_include_deploy_hashes', full_name='io.casperlabs.comm.gossiping.GetBlockChunkedRequest.only_include_deploy_hashes', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=611,
  serialized_end=783,
)


_STREAMDEPLOYSCHUNKEDREQUEST = _descriptor.Descriptor(
  name='StreamDeploysChunkedRequest',
  full_name='io.casperlabs.comm.gossiping.StreamDeploysChunkedRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='deploy_hashes', full_name='io.casperlabs.comm.gossiping.StreamDeploysChunkedRequest.deploy_hashes', index=0,
      number=1, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='chunk_size', full_name='io.casperlabs.comm.gossiping.StreamDeploysChunkedRequest.chunk_size', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='accepted_compression_algorithms', full_name='io.casperlabs.comm.gossiping.StreamDeploysChunkedRequest.accepted_compression_algorithms', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=785,
  serialized_end=898,
)


_GETGENESISCANDIDATEREQUEST = _descriptor.Descriptor(
  name='GetGenesisCandidateRequest',
  full_name='io.casperlabs.comm.gossiping.GetGenesisCandidateRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=900,
  serialized_end=928,
)


_ADDAPPROVALREQUEST = _descriptor.Descriptor(
  name='AddApprovalRequest',
  full_name='io.casperlabs.comm.gossiping.AddApprovalRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='block_hash', full_name='io.casperlabs.comm.gossiping.AddApprovalRequest.block_hash', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='approval', full_name='io.casperlabs.comm.gossiping.AddApprovalRequest.approval', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=930,
  serialized_end=1030,
)


_CHUNK_HEADER = _descriptor.Descriptor(
  name='Header',
  full_name='io.casperlabs.comm.gossiping.Chunk.Header',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='compression_algorithm', full_name='io.casperlabs.comm.gossiping.Chunk.Header.compression_algorithm', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='content_length', full_name='io.casperlabs.comm.gossiping.Chunk.Header.content_length', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='original_content_length', full_name='io.casperlabs.comm.gossiping.Chunk.Header.original_content_length', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1120,
  serialized_end=1216,
)

_CHUNK = _descriptor.Descriptor(
  name='Chunk',
  full_name='io.casperlabs.comm.gossiping.Chunk',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='io.casperlabs.comm.gossiping.Chunk.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='data', full_name='io.casperlabs.comm.gossiping.Chunk.data', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_CHUNK_HEADER, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='content', full_name='io.casperlabs.comm.gossiping.Chunk.content',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=1033,
  serialized_end=1227,
)


_STREAMDAGSLICEBLOCKSUMMARIESREQUEST = _descriptor.Descriptor(
  name='StreamDagSliceBlockSummariesRequest',
  full_name='io.casperlabs.comm.gossiping.StreamDagSliceBlockSummariesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='start_rank', full_name='io.casperlabs.comm.gossiping.StreamDagSliceBlockSummariesRequest.start_rank', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='end_rank', full_name='io.casperlabs.comm.gossiping.StreamDagSliceBlockSummariesRequest.end_rank', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1229,
  serialized_end=1304,
)

_NEWBLOCKSREQUEST.fields_by_name['sender'].message_type = node__pb2._NODE
_NEWDEPLOYSREQUEST.fields_by_name['sender'].message_type = node__pb2._NODE
_ADDAPPROVALREQUEST.fields_by_name['approval'].message_type = consensus__pb2._APPROVAL
_CHUNK_HEADER.containing_type = _CHUNK
_CHUNK.fields_by_name['header'].message_type = _CHUNK_HEADER
_CHUNK.oneofs_by_name['content'].fields.append(
  _CHUNK.fields_by_name['header'])
_CHUNK.fields_by_name['header'].containing_oneof = _CHUNK.oneofs_by_name['content']
_CHUNK.oneofs_by_name['content'].fields.append(
  _CHUNK.fields_by_name['data'])
_CHUNK.fields_by_name['data'].containing_oneof = _CHUNK.oneofs_by_name['content']
DESCRIPTOR.message_types_by_name['NewBlocksRequest'] = _NEWBLOCKSREQUEST
DESCRIPTOR.message_types_by_name['NewBlocksResponse'] = _NEWBLOCKSRESPONSE
DESCRIPTOR.message_types_by_name['NewDeploysRequest'] = _NEWDEPLOYSREQUEST
DESCRIPTOR.message_types_by_name['NewDeploysResponse'] = _NEWDEPLOYSRESPONSE
DESCRIPTOR.message_types_by_name['StreamBlockSummariesRequest'] = _STREAMBLOCKSUMMARIESREQUEST
DESCRIPTOR.message_types_by_name['StreamDeploySummariesRequest'] = _STREAMDEPLOYSUMMARIESREQUEST
DESCRIPTOR.message_types_by_name['StreamAncestorBlockSummariesRequest'] = _STREAMANCESTORBLOCKSUMMARIESREQUEST
DESCRIPTOR.message_types_by_name['StreamLatestMessagesRequest'] = _STREAMLATESTMESSAGESREQUEST
DESCRIPTOR.message_types_by_name['GetBlockChunkedRequest'] = _GETBLOCKCHUNKEDREQUEST
DESCRIPTOR.message_types_by_name['StreamDeploysChunkedRequest'] = _STREAMDEPLOYSCHUNKEDREQUEST
DESCRIPTOR.message_types_by_name['GetGenesisCandidateRequest'] = _GETGENESISCANDIDATEREQUEST
DESCRIPTOR.message_types_by_name['AddApprovalRequest'] = _ADDAPPROVALREQUEST
DESCRIPTOR.message_types_by_name['Chunk'] = _CHUNK
DESCRIPTOR.message_types_by_name['StreamDagSliceBlockSummariesRequest'] = _STREAMDAGSLICEBLOCKSUMMARIESREQUEST
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

NewBlocksRequest = _reflection.GeneratedProtocolMessageType('NewBlocksRequest', (_message.Message,), {
  'DESCRIPTOR' : _NEWBLOCKSREQUEST,
  '__module__' : 'gossiping_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.comm.gossiping.NewBlocksRequest)
  })
_sym_db.RegisterMessage(NewBlocksRequest)

NewBlocksResponse = _reflection.GeneratedProtocolMessageType('NewBlocksResponse', (_message.Message,), {
  'DESCRIPTOR' : _NEWBLOCKSRESPONSE,
  '__module__' : 'gossiping_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.comm.gossiping.NewBlocksResponse)
  })
_sym_db.RegisterMessage(NewBlocksResponse)

NewDeploysRequest = _reflection.GeneratedProtocolMessageType('NewDeploysRequest', (_message.Message,), {
  'DESCRIPTOR' : _NEWDEPLOYSREQUEST,
  '__module__' : 'gossiping_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.comm.gossiping.NewDeploysRequest)
  })
_sym_db.RegisterMessage(NewDeploysRequest)

NewDeploysResponse = _reflection.GeneratedProtocolMessageType('NewDeploysResponse', (_message.Message,), {
  'DESCRIPTOR' : _NEWDEPLOYSRESPONSE,
  '__module__' : 'gossiping_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.comm.gossiping.NewDeploysResponse)
  })
_sym_db.RegisterMessage(NewDeploysResponse)

StreamBlockSummariesRequest = _reflection.GeneratedProtocolMessageType('StreamBlockSummariesRequest', (_message.Message,), {
  'DESCRIPTOR' : _STREAMBLOCKSUMMARIESREQUEST,
  '__module__' : 'gossiping_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.comm.gossiping.StreamBlockSummariesRequest)
  })
_sym_db.RegisterMessage(StreamBlockSummariesRequest)

StreamDeploySummariesRequest = _reflection.GeneratedProtocolMessageType('StreamDeploySummariesRequest', (_message.Message,), {
  'DESCRIPTOR' : _STREAMDEPLOYSUMMARIESREQUEST,
  '__module__' : 'gossiping_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.comm.gossiping.StreamDeploySummariesRequest)
  })
_sym_db.RegisterMessage(StreamDeploySummariesRequest)

StreamAncestorBlockSummariesRequest = _reflection.GeneratedProtocolMessageType('StreamAncestorBlockSummariesRequest', (_message.Message,), {
  'DESCRIPTOR' : _STREAMANCESTORBLOCKSUMMARIESREQUEST,
  '__module__' : 'gossiping_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.comm.gossiping.StreamAncestorBlockSummariesRequest)
  })
_sym_db.RegisterMessage(StreamAncestorBlockSummariesRequest)

StreamLatestMessagesRequest = _reflection.GeneratedProtocolMessageType('StreamLatestMessagesRequest', (_message.Message,), {
  'DESCRIPTOR' : _STREAMLATESTMESSAGESREQUEST,
  '__module__' : 'gossiping_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.comm.gossiping.StreamLatestMessagesRequest)
  })
_sym_db.RegisterMessage(StreamLatestMessagesRequest)

GetBlockChunkedRequest = _reflection.GeneratedProtocolMessageType('GetBlockChunkedRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETBLOCKCHUNKEDREQUEST,
  '__module__' : 'gossiping_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.comm.gossiping.GetBlockChunkedRequest)
  })
_sym_db.RegisterMessage(GetBlockChunkedRequest)

StreamDeploysChunkedRequest = _reflection.GeneratedProtocolMessageType('StreamDeploysChunkedRequest', (_message.Message,), {
  'DESCRIPTOR' : _STREAMDEPLOYSCHUNKEDREQUEST,
  '__module__' : 'gossiping_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.comm.gossiping.StreamDeploysChunkedRequest)
  })
_sym_db.RegisterMessage(StreamDeploysChunkedRequest)

GetGenesisCandidateRequest = _reflection.GeneratedProtocolMessageType('GetGenesisCandidateRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETGENESISCANDIDATEREQUEST,
  '__module__' : 'gossiping_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.comm.gossiping.GetGenesisCandidateRequest)
  })
_sym_db.RegisterMessage(GetGenesisCandidateRequest)

AddApprovalRequest = _reflection.GeneratedProtocolMessageType('AddApprovalRequest', (_message.Message,), {
  'DESCRIPTOR' : _ADDAPPROVALREQUEST,
  '__module__' : 'gossiping_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.comm.gossiping.AddApprovalRequest)
  })
_sym_db.RegisterMessage(AddApprovalRequest)

Chunk = _reflection.GeneratedProtocolMessageType('Chunk', (_message.Message,), {

  'Header' : _reflection.GeneratedProtocolMessageType('Header', (_message.Message,), {
    'DESCRIPTOR' : _CHUNK_HEADER,
    '__module__' : 'gossiping_pb2'
    # @@protoc_insertion_point(class_scope:io.casperlabs.comm.gossiping.Chunk.Header)
    })
  ,
  'DESCRIPTOR' : _CHUNK,
  '__module__' : 'gossiping_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.comm.gossiping.Chunk)
  })
_sym_db.RegisterMessage(Chunk)
_sym_db.RegisterMessage(Chunk.Header)

StreamDagSliceBlockSummariesRequest = _reflection.GeneratedProtocolMessageType('StreamDagSliceBlockSummariesRequest', (_message.Message,), {
  'DESCRIPTOR' : _STREAMDAGSLICEBLOCKSUMMARIESREQUEST,
  '__module__' : 'gossiping_pb2'
  # @@protoc_insertion_point(class_scope:io.casperlabs.comm.gossiping.StreamDagSliceBlockSummariesRequest)
  })
_sym_db.RegisterMessage(StreamDagSliceBlockSummariesRequest)



_GOSSIPSERVICE = _descriptor.ServiceDescriptor(
  name='GossipService',
  full_name='io.casperlabs.comm.gossiping.GossipService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=1307,
  serialized_end=2702,
  methods=[
  _descriptor.MethodDescriptor(
    name='NewBlocks',
    full_name='io.casperlabs.comm.gossiping.GossipService.NewBlocks',
    index=0,
    containing_service=None,
    input_type=_NEWBLOCKSREQUEST,
    output_type=_NEWBLOCKSRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='NewDeploys',
    full_name='io.casperlabs.comm.gossiping.GossipService.NewDeploys',
    index=1,
    containing_service=None,
    input_type=_NEWDEPLOYSREQUEST,
    output_type=_NEWDEPLOYSRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='StreamAncestorBlockSummaries',
    full_name='io.casperlabs.comm.gossiping.GossipService.StreamAncestorBlockSummaries',
    index=2,
    containing_service=None,
    input_type=_STREAMANCESTORBLOCKSUMMARIESREQUEST,
    output_type=consensus__pb2._BLOCKSUMMARY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='StreamLatestMessages',
    full_name='io.casperlabs.comm.gossiping.GossipService.StreamLatestMessages',
    index=3,
    containing_service=None,
    input_type=_STREAMLATESTMESSAGESREQUEST,
    output_type=consensus__pb2._BLOCK_JUSTIFICATION,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='StreamBlockSummaries',
    full_name='io.casperlabs.comm.gossiping.GossipService.StreamBlockSummaries',
    index=4,
    containing_service=None,
    input_type=_STREAMBLOCKSUMMARIESREQUEST,
    output_type=consensus__pb2._BLOCKSUMMARY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='StreamDeploySummaries',
    full_name='io.casperlabs.comm.gossiping.GossipService.StreamDeploySummaries',
    index=5,
    containing_service=None,
    input_type=_STREAMDEPLOYSUMMARIESREQUEST,
    output_type=consensus__pb2._DEPLOYSUMMARY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetBlockChunked',
    full_name='io.casperlabs.comm.gossiping.GossipService.GetBlockChunked',
    index=6,
    containing_service=None,
    input_type=_GETBLOCKCHUNKEDREQUEST,
    output_type=_CHUNK,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='StreamDeploysChunked',
    full_name='io.casperlabs.comm.gossiping.GossipService.StreamDeploysChunked',
    index=7,
    containing_service=None,
    input_type=_STREAMDEPLOYSCHUNKEDREQUEST,
    output_type=_CHUNK,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetGenesisCandidate',
    full_name='io.casperlabs.comm.gossiping.GossipService.GetGenesisCandidate',
    index=8,
    containing_service=None,
    input_type=_GETGENESISCANDIDATEREQUEST,
    output_type=consensus__pb2._GENESISCANDIDATE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='AddApproval',
    full_name='io.casperlabs.comm.gossiping.GossipService.AddApproval',
    index=9,
    containing_service=None,
    input_type=_ADDAPPROVALREQUEST,
    output_type=empty__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='StreamDagSliceBlockSummaries',
    full_name='io.casperlabs.comm.gossiping.GossipService.StreamDagSliceBlockSummaries',
    index=10,
    containing_service=None,
    input_type=_STREAMDAGSLICEBLOCKSUMMARIESREQUEST,
    output_type=consensus__pb2._BLOCKSUMMARY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_GOSSIPSERVICE)

DESCRIPTOR.services_by_name['GossipService'] = _GOSSIPSERVICE

# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
from typing import Dict

from casperlabs_client import CasperLabsClient
from casperlabs_client.decorators import guarded_command


NAME: str = "send-deploy"
HELP: str = (
    "Deploy a smart contract source file to Casper on an existing running node. "
    "The deploy will be packaged and sent as a block to the network depending on "
    "the configuration of the Casper instance."
)
OPTIONS = [
    [
        ("-i", "--deploy-path"),
        dict(required=False, default=None, help="Path to the file with signed deploy."),
    ]
]


@guarded_command
def method(casperlabs_client: CasperLabsClient, args: Dict):
    deploy_hash = casperlabs_client.send_deploy(deploy_file=args.get("deploy_path"))
    print(f"Success! Deploy {deploy_hash} deployed")



# -*- coding: utf-8 -*-
import logging

import grpc

from casperlabs_client.decorators import retry_unary, retry_stream


class InsecureGRPCService:
    def __init__(self, host, port, serviceStub):
        self.address = f"{host}:{port}"
        self.serviceStub = serviceStub

    def __getattr__(self, name):

        logging.debug(
            f"Creating insecure connection to {self.address} ({self.serviceStub})"
        )

        @retry_unary
        def unary_unary(*args):
            logging.debug(
                f"Insecure {self.address} ({self.serviceStub}): {name} {list(args)}"
            )
            with grpc.insecure_channel(self.address) as channel:
                return getattr(self.serviceStub(channel), name)(*args)

        @retry_stream
        def unary_stream(*args):
            logging.debug(
                f"Insecure {self.address} ({self.serviceStub}): {name} {list(args)}"
            )
            with grpc.insecure_channel(self.address) as channel:
                yield from getattr(self.serviceStub(channel), name[: -len("_stream")])(
                    *args
                )

        return name.endswith("_stream") and unary_stream or unary_unary



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import functools
from pathlib import Path
import time
import warnings
from typing import List, Union

# Monkey patching of google.protobuf.text_encoding.CEscape
# to get keys and signatures in hex when printed
import google.protobuf.text_format

from .consts import ED25519_KEY_ALGORITHM
from .reformat import optional_base64_base16_to_bytes

CEscape = google.protobuf.text_format.text_encoding.CEscape


def _hex(text, as_utf8):
    try:
        if len(text) in (32, 64, 20):
            return text.hex()
        else:
            return CEscape(text, as_utf8)
    except TypeError:
        return CEscape(text, as_utf8)


google.protobuf.text_format.text_encoding.CEscape = _hex

# Hack to fix the relative imports problems with grpc #
import sys

sys.path.append(str(Path(__file__).resolve().parents[1]))
# end of hack #

from . import io, common, vdag, abi, key_holders
from .insecure_grpc_service import InsecureGRPCService
from .secure_grpc_service import SecureGRPCService
from .query_state import key_variant
from .deploy import DeployData, sign_deploy

from grpc._channel import _Rendezvous

# ~/CasperLabs/protobuf/io/casperlabs/node/api/control.proto
from . import control_pb2_grpc, consts, crypto

# ~/CasperLabs/protobuf/io/casperlabs/node/api/casper.proto
from . import casper_pb2 as casper
from . import casper_pb2_grpc

# ~/CasperLabs/protobuf/io/casperlabs/casper/consensus/info.proto
from . import info_pb2 as info

# ~/CasperLabs/protobuf/io/casperlabs/node/api/diagnostics.proto
from . import diagnostics_pb2_grpc
from . import empty_pb2


class InternalError(Exception):
    """
    The only exception that API calls can throw.
    Internal errors like gRPC exceptions will be caught
    and this exception thrown instead, so the user does
    not have to worry about handling any other exceptions.
    """

    # TODO: Is there a reason we need to hide error types?
    def __init__(self, status="", details=""):
        super(InternalError, self).__init__()
        self.status = status
        self.details = details

    def __str__(self):
        return f"{self.status}: {self.details}"


def api(function):
    """
    Decorator of API functions that protects user code from
    unknown exceptions raised by gRPC or internal API errors.
    It will catch all exceptions and throw InternalError.

    :param function: function to be decorated
    :return: decorated function
    """

    @functools.wraps(function)
    def wrapper(*args, **kwargs):
        try:
            return function(*args, **kwargs)
        except (SyntaxError, TypeError, InternalError):
            raise
        except _Rendezvous as e:
            raise InternalError(str(e.code()), e.details())
        except Exception as e:
            raise InternalError(details=str(e)) from e

    return wrapper


class CasperLabsClient:
    """
    gRPC CasperLabs client.
    """

    def __init__(
        self,
        host: str = consts.DEFAULT_HOST,
        port: int = consts.DEFAULT_PORT,
        port_internal: int = consts.DEFAULT_INTERNAL_PORT,
        node_id: str = None,
        certificate_file: str = None,
    ):
        """
        CasperLabs client's constructor.

        :param host: Hostname or IP of node on which gRPC service is running
        :param port: Port used for external gRPC API
        :param port_internal: Port used for internal gRPC API
        :param node_id: node_id of the node, for gRPC encryption
        :param certificate_file: Certificate file for TLS
        """
        self.host = host
        self.port = port
        self.port_internal = port_internal
        self.node_id = node_id
        self.certificate_file = certificate_file

        if node_id:
            self.casper_service = SecureGRPCService(
                host, port, casper_pb2_grpc.CasperServiceStub, node_id, certificate_file
            )
            self.control_service = SecureGRPCService(
                # We currently assume that if node_id is given then
                # we get certificate_file too. This is unlike in the Scala client
                # where node_id is all that's needed for configuring secure connection.
                # The reason for this is that currently it doesn't seem to be possible
                # to open a secure grpc connection in Python without supplying any
                # certificate on the client side.
                host,
                port_internal,
                control_pb2_grpc.ControlServiceStub,
                node_id,
                certificate_file,
            )
            self.diagnostics_service = SecureGRPCService(
                host,
                port,
                diagnostics_pb2_grpc.DiagnosticsStub,
                node_id,
                certificate_file,
            )

        else:
            self.casper_service = InsecureGRPCService(
                host, port, casper_pb2_grpc.CasperServiceStub
            )
            self.control_service = InsecureGRPCService(
                host, port_internal, control_pb2_grpc.ControlServiceStub
            )
            self.diagnostics_service = InsecureGRPCService(
                host, port, diagnostics_pb2_grpc.DiagnosticsStub
            )

    @api
    def make_deploy(
        self,
        from_addr: Union[bytes, str] = None,
        public_key: Union[Path, str] = None,
        public_key_hex: str = None,
        algorithm: str = ED25519_KEY_ALGORITHM,
        payment: Union[Path, str] = None,
        payment_args: bytes = None,
        payment_amount: int = None,
        payment_hash: bytes = None,
        payment_name: str = None,
        payment_package_hash: bytes = None,
        payment_package_name: str = None,
        payment_entry_point: str = None,
        payment_version: int = None,
        session: Union[Path, str] = None,
        session_args: bytes = None,
        session_hash: bytes = None,
        session_name: str = None,
        session_package_hash: bytes = None,
        session_package_name: str = None,
        session_entry_point: str = None,
        session_version: int = None,
        ttl_millis: int = 0,
        dependencies: list = None,
        chain_name: str = None,
        transfer_args: bytes = None,
    ):
        """
        Create a protobuf deploy object.

        :param from_addr: Account hash to use for deployment, generated from private_key or private_key_hex if omitted.
                          Default payment will use the main purse of this account, unless custom payment is provided.
        :param public_key: Path to a file with public key. Assumed to be ed25519 unless algorithm is given.
        :param public_key_hex: Public key (base16/hex encoded). Assumed to be ed25519 unless algorithm is given.
        :param algorithm: Algorithm used for generating keys, defaults to ed25519. See consts.SUPPORTED_KEY_ALGORITHMS
        :param payment: Path to the file with payment code as compiled wasm.
        :param payment_args: List of encoded arguments of payment contract
        :param payment_amount: Amount to be used with standard payment. Will pull from main purse.
        :param payment_hash: Hash (base16/hex encoded) of the stored contract to be called for payment.
        :param payment_name: Name of the stored contract to be called for payment.
        :param payment_package_hash Hash of the package to be called in the payment.
        :param payment_package_name Name of the package to be called in the payment.
        :param payment_entry_point: Name of Entrypoint in contract or package to call.
                                    Defaults to `call` if not given.
        :param payment_version: Version of the payment contract or package to use, defaults to latest if omitted
        :param session: Path to the file with session code as compiled wasm.
        :param session_args: List of encoded arguments of session contract
        :param session_hash: Hash (base16/hex encoded) of the stored contract (associated with the executing account)
                             to be called in the session.
        :param session_name: Name of the stored contract (associated with the executing account) to be called in the
                             session.
        :param session_package_hash Hash of the package to be called in the session.
        :param session_package_name Name of the package to be called in the session.
        :param session_entry_point: Name of Entrypoint in contract to call. Defaults to `call` if not given.
        :param session_version: Version of the session contract or package to use, defaults to latest if omitted
        :param ttl_millis: Time to live. Time (in milliseconds) that the deploy will remain valid for.
        :param dependencies: List of deploy hashes (hex/base16 encoded) which must be executed before this deploy.
        :param chain_name: Name of the chain to optionally restrict the deploy from being accidentally included anywhere else.
        :param transfer_args: Should not be used, call transfer method instead.
        :return: protobuf deploy object which must be signed before sending
        """
        deploy_data = DeployData.from_args(
            dict(
                from_addr=from_addr,
                public_key=public_key,
                public_key_hex=public_key_hex,
                algorithm=algorithm,
                payment=payment,
                payment_args=payment_args,
                payment_amount=payment_amount,
                payment_hash=payment_hash,
                payment_name=payment_name,
                payment_package_hash=payment_package_hash,
                payment_package_name=payment_package_name,
                payment_entry_point=payment_entry_point,
                payment_version=payment_version,
                session=session,
                session_args=session_args,
                session_hash=session_hash,
                session_name=session_name,
                session_package_hash=session_package_hash,
                session_package_name=session_package_name,
                session_entry_point=session_entry_point,
                session_version=session_version,
                ttl_millis=ttl_millis,
                dependencies=dependencies,
                chain_name=chain_name,
                transfer_args=transfer_args,
            )
        )
        return deploy_data.make_protobuf()

    @api
    def sign_deploy(
        self,
        private_key_pem_file: Union[str, Path] = None,
        private_key_hex: str = None,
        algorithm: str = ED25519_KEY_ALGORITHM,
        key_holder=None,
        deploy: bytes = None,
        deploy_file: Union[str, Path] = None,
    ):
        """
        Sign a deploy with the given key.  Source of deploy may be deploy object or file containing deploy.
        Must provide one of the private key values or a key_holder object.
        Must provide one of the deploy values.

        :param private_key_pem_file: File containing Private key
        :param private_key_hex: Private key in hex
        :param algorithm: Algorithm used for generating keys, defaults to ed25519. See consts.SUPPORTED_KEY_ALGORITHMS
        :param key_holder: KeyHolder object as alternative to pem file and algorithm
        :param deploy: Deploy as object
        :param deploy_file: File containing deploy

        :return: signed deploy object
        """
        if deploy is None:
            if deploy_file is None:
                raise ValueError("Must have either `deploy` or `deploy_file`")
            deploy = io.read_deploy_file(deploy_file)
        private_key = bytes.fromhex(private_key_hex) if private_key_hex else None
        if not any((private_key_pem_file, private_key, key_holder)):
            raise ValueError(
                "Must have either `private_key_pem_file`, `private_key_hex`, or `key_holder`"
            )
        if not key_holder:
            key_holder = key_holders.key_holder_object(
                algorithm,
                private_key_pem_path=private_key_pem_file,
                private_key=private_key,
            )
        return sign_deploy(deploy, key_holder)

    @api
    def deploy(
        self,
        from_addr: Union[bytes, str] = None,
        private_key: Union[Path, str] = None,
        private_key_hex: str = None,
        algorithm: str = ED25519_KEY_ALGORITHM,
        payment: Union[Path, str] = None,
        payment_args: bytes = None,
        payment_amount: int = None,
        payment_hash: bytes = None,
        payment_name: str = None,
        payment_package_hash: bytes = None,
        payment_package_name: str = None,
        payment_entry_point: str = None,
        payment_version: int = None,
        session: Union[Path, str] = None,
        session_args: bytes = None,
        session_hash: bytes = None,
        session_name: str = None,
        session_package_hash: bytes = None,
        session_package_name: str = None,
        session_entry_point: str = None,
        session_version: int = None,
        ttl_millis: int = 0,
        dependencies=None,
        chain_name: str = None,
        transfer_args: bytes = None,
    ) -> str:
        """
        Create a deploy on the network.

        This is equivalent to performing `make_deploy`, `sign_deploy`, and `send_deploy`.

        :param from_addr: Account hash to use for deployment, generated from private_key or private_key_hex if omitted.
                          Default payment will use the main purse of this account, unless custom payment is provided.
        :param private_key: Path to a file with private key. Assumed to be ed25519 unless algorithm is given.
        :param private_key_hex: Private key (base16/hex encoded). Assumed to be ed25519 unless algorithm is given.
        :param algorithm: Algorithm used for generating keys, defaults to ed25519. See consts.SUPPORTED_KEY_ALGORITHMS
        :param payment: Path to the file with payment code as compiled wasm.
        :param payment_args: List of encoded arguments of payment contract
        :param payment_amount: Amount to be used with standard payment. Will pull from main purse.
        :param payment_hash: Hash (base16/hex encoded) of the stored contract to be called for payment.
        :param payment_name: Name of the stored contract to be called for payment.
        :param payment_package_hash Hash of the package to be called in the payment.
        :param payment_package_name Name of the package to be called in the payment.
        :param payment_entry_point: Name of Entrypoint in contract or package to call.
                                    Defaults to `call` if not given.
        :param payment_version: Version of the payment contract or package to use, defaults to latest if omitted
        :param session: Path to the file with session code as compiled wasm.
        :param session_args: List of encoded arguments of session contract
        :param session_hash: Hash (base16/hex encoded) of the stored contract (associated with the executing account)
                             to be called in the session.
        :param session_name: Name of the stored contract (associated with the executing account) to be called in the
                             session.
        :param session_package_hash Hash of the package to be called in the session.
        :param session_package_name Name of the package to be called in the session.
        :param session_entry_point: Name of Entrypoint in contract to call. Defaults to `call` if not given.
        :param session_version: Version of the session contract or package to use, defaults to latest if omitted
        :param ttl_millis: Time to live. Time (in milliseconds) that the deploy will remain valid for.
        :param dependencies: List of deploy hashes (hex/base16 encoded) which must be executed before this deploy.
        :param chain_name: Name of the chain to optionally restrict the deploy from being accidentally included anywhere else.
        :param transfer_args: Should not be used, call transfer method instead.
        :return: deploy_hash (base16/hex encoded)
        """

        # Using from_args to get validation on build of object
        deploy_data = DeployData.from_args(
            dict(
                from_addr=from_addr,
                private_key=private_key,
                private_key_hex=private_key_hex,
                algorithm=algorithm,
                payment=payment,
                payment_args=payment_args,
                payment_amount=payment_amount,
                payment_hash=payment_hash,
                payment_name=payment_name,
                payment_package_hash=payment_package_hash,
                payment_package_name=payment_package_name,
                payment_entry_point=payment_entry_point,
                payment_version=payment_version,
                session=session,
                session_args=session_args,
                session_hash=session_hash,
                session_name=session_name,
                session_package_hash=session_package_hash,
                session_package_name=session_package_name,
                session_entry_point=session_entry_point,
                session_version=session_version,
                transfer_args=transfer_args,
                ttl_millis=ttl_millis,
                dependencies=dependencies,
                chain_name=chain_name,
            )
        )

        deploy_proto = deploy_data.make_protobuf()

        signed_deploy = self.sign_deploy(
            deploy=deploy_proto, key_holder=deploy_data.key_holder
        )
        self.send_deploy(signed_deploy)
        return signed_deploy.deploy_hash.hex()

    @api
    def transfer(
        self,
        amount: int,
        target_account: Union[str, bytes] = None,
        target_purse: Union[str, bytes] = None,
        source_purse: Union[str, bytes] = None,
        from_addr: Union[str, bytes] = None,
        private_key: str = None,
        private_key_hex: str = None,
        algorithm: str = ED25519_KEY_ALGORITHM,
        payment: str = None,
        payment_args: bytes = None,
        payment_amount: int = None,
        payment_hash: bytes = None,
        payment_name: str = None,
        payment_package_hash: bytes = None,
        payment_package_name: str = None,
        payment_entry_point: str = None,
        payment_version: int = None,
        ttl_millis: int = 0,
        dependencies=None,
        chain_name: str = None,
    ) -> str:
        """
        Transfer between account purses.

        :param amount: Amount to transfer
        :param target_account: Target account hash (base16/hex encoded), will use main purse.
        :param target_purse: URef of target purse (base16/hex encoded).
        :param source_purse: URef of source purse (base16/hex encoded). If omitted, will use main purse of account.
        :param from_addr: Account hash to use for deployment, generated from private_key or private_key_hex if omitted.
                          Default payment will use the main purse of this account, unless custom payment is provided.
        :param private_key: Path to a file with private key. Assumed to be ed25519 unless algorithm is given.
        :param private_key_hex: Private key (base16/hex encoded). Assumed to be ed25519 unless algorithm is given.
        :param algorithm: Algorithm used for generating keys, defaults to ed25519. See consts.SUPPORTED_KEY_ALGORITHMS
        :param payment: Path to the file with payment code as compiled wasm.
        :param payment_args: List of encoded arguments of payment contract
        :param payment_amount: Amount to be used with standard payment. Will pull from main purse.
        :param payment_hash: Hash (base16/hex encoded) of the stored contract to be called for payment.
        :param payment_name: Name of the stored contract to be called for payment.
        :param payment_package_hash Hash of the package to be called in the payment.
        :param payment_package_name Name of the package to be called in the payment.
        :param payment_entry_point: Name of Entrypoint in contract or package to call.
                                    Defaults to `call` if not given.
        :param payment_version: Version of the payment contract or package to use, defaults to latest if omitted
        :param ttl_millis: Time to live. Time (in milliseconds) that the deploy will remain valid for.
        :param dependencies: List of deploy hashes (hex/base16 encoded) which must be executed before this deploy.
        :param chain_name: Name of the chain to optionally restrict the deploy from being accidentally included anywhere else.
        :return: deploy_hash (base16/hex encoded) of transfer
        """
        if len(list(filter(None, (target_purse, target_account)))) != 1:
            raise ValueError("must include either target_account or target_purse")

        target_account_bytes = optional_base64_base16_to_bytes(
            target_account, "target_account"
        )
        target_purse_bytes = optional_base64_base16_to_bytes(
            target_purse, "target_purse"
        )
        source_purse_bytes = optional_base64_base16_to_bytes(
            source_purse, "source_purse"
        )

        target_send_bytes = target_account_bytes or target_purse_bytes

        arg_list = [
            abi.ABI.fixed_list("target", target_send_bytes),
            abi.ABI.u512("amount", amount),
        ]
        if source_purse_bytes:
            arg_list.append(abi.ABI.fixed_list("source", source_purse_bytes))

        transfer_args = abi.ABI.args(arg_list)
        return self.deploy(
            from_addr=from_addr,
            private_key=private_key,
            private_key_hex=private_key_hex,
            algorithm=algorithm,
            payment=payment,
            payment_args=payment_args,
            payment_amount=payment_amount,
            payment_hash=payment_hash,
            payment_name=payment_name,
            payment_package_hash=payment_package_hash,
            payment_package_name=payment_package_name,
            payment_entry_point=payment_entry_point,
            payment_version=payment_version,
            ttl_millis=ttl_millis,
            dependencies=dependencies,
            chain_name=chain_name,
            transfer_args=transfer_args,
        )

    @api
    def send_deploy(self, deploy=None, deploy_file=None) -> str:
        """
        Sends deploy to network from either deploy protobuf object or file.

        :param deploy: Protobuf deploy object
        :param deploy_file: File holding Protobuf deploy object
        :return: deploy hash (base16/hex encoded)
        """
        if deploy is None:
            if deploy_file is None:
                raise ValueError("Must have either deploy or deploy_file.")
            deploy = io.read_deploy_file(deploy_file)
        self.casper_service.Deploy(casper.DeployRequest(deploy=deploy))
        return deploy.deploy_hash.hex()

    @api
    def show_blocks(self, depth: int = 1, max_rank: int = 0, full_view: bool = True):
        """
        Get slices of the DAG, going backwards, rank by rank.

        :param depth: How many of the top ranks of the DAG to show.
        :param max_rank: Maximum rank to go back from. 0 means go from the current tip of the DAG.
        :param full_view: Full view if True, otherwise basic.
        :return: Generator of block info objects.
        """
        yield from self.casper_service.StreamBlockInfos_stream(
            casper.StreamBlockInfosRequest(
                depth=depth,
                max_rank=max_rank,
                view=(
                    full_view and info.BlockInfo.View.FULL or info.BlockInfo.View.BASIC
                ),
            )
        )

    @api
    def showBlocks(self, depth: int = 1, max_rank=0, full_view=True):
        """DEPRECATED: Call `show_blocks`"""
        warnings.warn(
            "showBlocks is deprecated and replaced with show_blocks.",
            DeprecationWarning,
        )
        for block_info_request in self.show_blocks(depth, max_rank, full_view):
            yield block_info_request

    @api
    def show_block(self, block_hash_base16: str, full_view=True):
        """
        Returns object describing a block on the network.

        :param block_hash_base16: hash of the block to be retrieved
        :param full_view: full view if True, otherwise basic
        :return: object representing the retrieved block
        """
        return self.casper_service.GetBlockInfo(
            casper.GetBlockInfoRequest(
                block_hash_base16=block_hash_base16,
                view=(
                    full_view and info.BlockInfo.View.FULL or info.BlockInfo.View.BASIC
                ),
            )
        )

    @api
    def showBlock(self, block_hash_base16: str, full_view=True):
        """DEPRECATED: call `show_block`"""
        warnings.warn(
            "showBlock is deprecated and replaced with show_block.", DeprecationWarning
        )
        return self.show_block(block_hash_base16, full_view)

    @api
    def visualize_dag(
        self,
        depth: int,
        out: str = None,
        show_justification_lines: bool = False,
        stream: str = None,
        delay_in_seconds=consts.VISUALIZE_DAG_STREAM_DELAY,
    ):
        """
        Visualize network dag.

        :param depth: Depth in terms of block height.
        :param out: Output image filename, outputs to stdout if not specified.
                    Must end with one of the png, svg, svg_standalone, xdot, plain, plain_ext, ps, ps2, json, json0
        :param show_justification_lines: If justification lines should be shown.
        :param stream: Subscribe to changes. 'out' must be specified.
                       Valid values are 'single-output', 'multiple-outputs'
        :param delay_in_seconds: Delay in seconds when polling for updates (streaming)
        :return: Yields generated DOT source or file name when out provided.
                 Generates endless stream of file names if stream is not None.
        """
        block_infos = list(self.show_blocks(depth, full_view=False))
        dot_dag_description = vdag.generate_dot(block_infos, show_justification_lines)
        if not out:
            yield dot_dag_description
            return

        parts = out.split(".")
        file_format = parts[-1]
        file_name_base = ".".join(parts[:-1])

        iteration = -1

        def file_name():
            nonlocal iteration, file_format
            if not stream or stream == "single-output":
                return out
            else:
                iteration += 1
                return f"{file_name_base}_{iteration}.{file_format}"

        yield vdag.call_dot(dot_dag_description, file_name(), file_format)
        previous_block_hashes = set(b.summary.block_hash for b in block_infos)
        while stream:
            time.sleep(delay_in_seconds)
            block_infos = list(self.show_blocks(depth, full_view=False))
            block_hashes = set(b.summary.block_hash for b in block_infos)
            if block_hashes != previous_block_hashes:
                dot_dag_description = vdag.generate_dot(
                    block_infos, show_justification_lines
                )
                yield vdag.call_dot(dot_dag_description, file_name(), file_format)
                previous_block_hashes = block_hashes

    @api
    def visualizeDag(
        self,
        depth: int,
        out: str = None,
        show_justification_lines: bool = False,
        stream: str = None,
        delay_in_seconds=5,
    ):
        """DEPRECATED: call `visualize_dag`"""
        warnings.warn(
            "visualizeDag is deprecated and replaced with visualize_dag.",
            DeprecationWarning,
        )
        for output in self.visualize_dag(
            depth, out, show_justification_lines, stream, delay_in_seconds
        ):
            yield output

    @api
    def query_state(self, block_hash: str, key: str, path: str, key_type: str):
        """
        Query a value in the global state.

        :param block_hash: Hash of the block (base16/hex encoding) of which to query.
        :param key: Base key (base16/hex encoded).
        :param path: Path to the value to query. Must be of the form 'key1/key2/.../key[n]'
        :param key_type: Type of base key. Must be one of 'hash', 'uref', 'address' or 'local'.
                         For 'local' key type, 'key' value format is {seed}:{rest},
                         where both parts are base16/hex encoded.
        :return: QueryStateResponse object
        """
        q = casper.StateQuery(key_variant=key_variant(key_type), key_base16=key)
        q.path_segments.extend([name for name in path.split("/") if name])
        return self.casper_service.GetBlockState(
            casper.GetBlockStateRequest(block_hash_base16=block_hash, query=q)
        )

    @api
    def queryState(self, blockHash: str, key: str, path: str, keyType: str):
        """DEPRECATED: call `query_state`"""
        warnings.warn(
            "queryState is deprecated and replaced with query_state.",
            DeprecationWarning,
        )
        return self.query_state(blockHash, key, path, keyType)

    @api
    def keygen(
        self,
        directory: Union[Path, str],
        algorithm: str = ED25519_KEY_ALGORITHM,
        filename_prefix: str = consts.DEFAULT_KEY_FILENAME_PREFIX,
    ) -> None:
        """
        Generates account keys into existing directory.
        Existing files in directory will be overwritten.

        :param directory: output directory, must exist
        :param algorithm: Algorithm used for generating keys, defaults to ed25519. See consts.SUPPORTED_KEY_ALGORITHMS
        :param filename_prefix: Prefix to use for files, default is 'account'
        :return: None

        Generated files:
           {filename_prefix}-id           # Hash of public key to use in the system (base64 encoded)
           {filename_prefix}-id-hex       # Hash of public key to use in the system (base16/hex encoded)
           {filename_prefix}-pk           # Public key (base64 encoded)
           {filename_prefix}-pk-hex       # Public key (base16/hex encoded)
           {filename_prefix}-private.pem  # private key pem file
           {filename_prefix}-public.pem   # public key pen file
        """
        directory = Path(directory)
        if not directory.exists():
            raise ValueError(f"Destination directory: {directory} does not exists.")

        key_holder_generator = key_holders.class_from_algorithm(algorithm)
        key_holder = key_holder_generator.generate()
        key_holder.save_pem_files(directory, filename_prefix)
        key_holder.save_hex_base64_files(directory, filename_prefix)

    @api
    def balance(self, address: str, block_hash: str) -> int:
        """
        Return balance of the main purse of an account

        :param address: Public key hash address of account.
        :param block_hash: Hash of block from which to return balance.
        :return: purse balance as int
        """
        # TODO: Should add ability to get balance of other purse with URef
        value = self.query_state(block_hash, address, "", "address")
        try:
            account = value.account
        except AttributeError:
            raise InternalError(
                "balance", f"Expected Account type value under {address}."
            )

        purse_addr_hex = account.main_purse.uref.hex()

        balance_u_ref = self.query_state(block_hash, purse_addr_hex, "", "hash")
        balance_u_ref_hex = balance_u_ref.cl_value.value.key.uref.uref.hex()
        balance = self.query_state(block_hash, balance_u_ref_hex, "", "uref")
        balance_str_value = balance.cl_value.value.u512.value
        return int(balance_str_value)

    @api
    def show_deploy(
        self,
        deploy_hash: str,
        full_view: bool = False,
        wait_for_processed: bool = False,
        delay: int = consts.STATUS_CHECK_DELAY,
        timeout_seconds: int = consts.STATUS_TIMEOUT,
    ):
        """
        Retrieve information about a single deploy by hash.

        :param deploy_hash: Hash of deploy (base16/hex encoded).
        :param full_view: Show full view of deploy, default False
        :param wait_for_processed: Block return of method until deploy is not pending, default False.
        :param delay: Delay between status checks while waiting, default consts.STATUS_CHECK_DELAY
        :param timeout_seconds: Time to wait, default consts.STATUS_TIMEOUT
                                Exception is raised if still pending when timeout occurs.
        :return: deploy_info
        """
        start_time = time.time()
        while True:
            deploy_info = self.casper_service.GetDeployInfo(
                casper.GetDeployInfoRequest(
                    deploy_hash_base16=deploy_hash,
                    view=common.deploy_info_view(full_view),
                )
            )
            if (
                wait_for_processed
                and deploy_info.status.state == info.DeployInfo.State.PENDING
            ):
                if time.time() - start_time > timeout_seconds:
                    raise Exception(
                        f"Timed out waiting for deploy {deploy_hash} to be processed"
                    )
                time.sleep(delay)
                continue
            return deploy_info

    @api
    def showDeploy(
        self,
        deploy_hash_base16: str,
        full_view: bool = False,
        wait_for_processed: bool = False,
        delay: int = consts.STATUS_CHECK_DELAY,
        timeout_seconds: int = consts.STATUS_TIMEOUT,
    ):
        """DEPRECATED: call `show_deploy`"""
        warnings.warn(
            "showDeploy is deprecated and replaced with show_deploy.",
            DeprecationWarning,
        )
        return self.show_deploy(
            deploy_hash_base16, full_view, wait_for_processed, delay, timeout_seconds
        )

    @api
    def show_deploys(self, block_hash_base16: str, full_view=True):
        """
        Get the processed deploys within a block.
        """
        yield from self.casper_service.StreamBlockDeploys_stream(
            casper.StreamBlockDeploysRequest(
                block_hash_base16=block_hash_base16,
                view=common.deploy_info_view(full_view),
            )
        )

    @api
    def showDeploys(self, block_hash_base16: str, full_view=True):
        """DEPRECATED: use `show_deploys`"""
        warnings.warn(
            "showDeploys is deprecated and replaced with show_deploys.",
            DeprecationWarning,
        )
        for block_deploys in self.show_deploy(block_hash_base16, full_view):
            yield block_deploys

    @api
    def stream_events(
        self,
        all: bool = False,
        block_added: bool = False,
        block_finalized: bool = False,
        deploy_added: bool = False,
        deploy_discarded: bool = False,
        deploy_requeued: bool = False,
        deploy_processed: bool = False,
        deploy_finalized: bool = False,
        deploy_orphaned: bool = False,
        account_public_key_hashes: List[str] = None,
        deploy_hashes: List[str] = None,
        min_event_id: int = 0,
        max_event_id: int = 0,
    ):
        """
        Generator of events from a node.  Allows replay or subscription to all future events.

        :param all: subscribe to all events, ignores other event values passed.
        :param block_added: subscribe to block added event.
        :param block_finalized: subscribe to block finalized event.
        :param deploy_added: subscribe to deploy added event.
        :param deploy_discarded: subscribe to deploy discarded event.
        :param deploy_requeued: subscribe to deploy requeued event.
        :param deploy_processed: subscribe to deploy processed event.
        :param deploy_finalized: subscribe to deploy finalized event.
        :param deploy_orphaned: subscribe to deploy orphaned event.
        :param account_public_key_hashes: list of account public key hashes (hex/base16 encoded) to use as a filter.
        :param deploy_hashes: list of deploy_hashes (hex/base16 encoded) to use as a filter.
        :param min_event_id: Value of 0 will subscribe to future events.
                             Non-zero values, will replay all past events from that ID, without subscribing to new.
                             To catch up with events from the beginning, start from 1.
        :param max_event_id: Id to stop replaying past events.
        :return: generator

        .. see also::
            See StreamEventsRequest in
                https://github.com/CasperLabs/CasperLabs/protobuf/io/casperlabs/node/api/casper.proto
            for description of types of events.
        """
        if not any(
            (
                all,
                block_added,
                block_finalized,
                deploy_added,
                deploy_discarded,
                deploy_requeued,
                deploy_processed,
                deploy_finalized,
                deploy_orphaned,
            )
        ):
            raise ValueError("No events given to subscribe.")

        if all:
            block_added = True
            block_finalized = True
            deploy_added = True
            deploy_discarded = True
            deploy_requeued = True
            deploy_processed = True
            deploy_finalized = True
            deploy_orphaned = True

        if account_public_key_hashes is None:
            account_public_key_hashes = []
        if deploy_hashes is None:
            deploy_hashes = []

        deploy_filter = casper.StreamEventsRequest.DeployFilter(
            account_public_key_hashes=[
                bytes.fromhex(pk) for pk in account_public_key_hashes
            ],
            deploy_hashes=[bytes.fromhex(h) for h in deploy_hashes],
        )

        yield from self.casper_service.StreamEvents_stream(
            casper.StreamEventsRequest(
                block_added=block_added,
                block_finalized=block_finalized,
                deploy_added=deploy_added,
                deploy_discarded=deploy_discarded,
                deploy_requeued=deploy_requeued,
                deploy_processed=deploy_processed,
                deploy_finalized=deploy_finalized,
                deploy_orphaned=deploy_orphaned,
                deploy_filter=deploy_filter,
                min_event_id=min_event_id,
                max_event_id=max_event_id,
            )
        )

    @staticmethod
    @api
    def validator_keygen(directory: Union[Path, str]) -> None:
        """
        Generate validator and node keys.

        :param directory: Directory in which to create files. Must exist.

        Generated files:
           node-id               # node ID as in casperlabs://c0a6c82062461c9b7f9f5c3120f44589393edf31@<NODE ADDRESS>?protocol=40400&discovery=40404
                                 # derived from node.key.pem
           node.certificate.pem  # TLS certificate used for node-to-node interaction encryption
                                 # derived from node.key.pem
           node.key.pem          # secp256r1 private key
           validator-pk          # validator public key in base64 format
           validator-pk-hex      # validator public key in hex/base16
           validator-id          # validator account hash in base64 format
           validator-id-hex      # validator account hash in hex/base16
           validator-private.pem # ed25519 private key
           validator-public.pem  # ed25519 public key
        """
        directory = Path(directory)
        if not directory.exists():
            raise ValueError(f"Destination directory: {directory} does not exists.")

        node_private_path = directory / consts.NODE_PRIVATE_KEY_FILENAME
        node_cert_path = directory / consts.NODE_CERTIFICATE_FILENAME
        node_id_path = directory / consts.NODE_ID_FILENAME

        key_pair = key_holders.ED25519Key.generate()
        key_pair.save_pem_files(directory, consts.VALIDATOR_FILENAME_PREFIX)
        key_pair.save_hex_base64_files(directory, consts.VALIDATOR_FILENAME_PREFIX)

        private_key, public_key = crypto.generate_secp256r1_key_pair()
        node_cert, key_pem = crypto.generate_node_certificates(private_key, public_key)

        io.write_binary_file(node_private_path, key_pem)
        io.write_binary_file(node_cert_path, node_cert)
        io.write_file(node_id_path, crypto.node_public_address(public_key))

    @api
    def wait_for_deploy_processed(
        self,
        deploy_hash,
        on_error_raise=True,
        delay=consts.STATUS_CHECK_DELAY,
        timeout_seconds=consts.STATUS_TIMEOUT,
    ):
        """
        Block up to `timeout_seconds` while `info.DeployInfo.State` is PENDING.


        :param deploy_hash: base16 (hex) deploy hash to wait for processing
        :param on_error_raise: raise and exception if the execution of the deploy is_error
        :param delay: delay between checks in milliseconds
        :param timeout_seconds: time to raise exception to exit if state is still PENDING
        :return: result of deploy


        """
        start_time = time.time()
        while True:
            if time.time() - start_time > timeout_seconds:
                raise Exception(
                    f"Timed out waiting for deploy {deploy_hash} to be processed"
                )

            result = self.show_deploy(deploy_hash, full_view=False)
            if result.status.state != info.DeployInfo.State.PENDING:
                break
            time.sleep(delay)

        if on_error_raise:
            if len(result.processing_results) == 0:
                raise Exception(f"Deploy {deploy_hash} status: {result.status}")

            last_processing_result = result.processing_results[0]
            if last_processing_result.is_error:
                raise Exception(
                    f"Deploy {deploy_hash} execution error: {last_processing_result.error_message}"
                )
        return result

    @api
    def show_peers(self) -> List:
        """ Return list of nodes connected to current node. """
        return list(self.diagnostics_service.ListPeers(empty_pb2.Empty()).peers)

    @api
    def account_hash(
        self, algorithm: str, public_key: bytes = None, public_key_pem_path: str = None
    ) -> bytes:
        """ Create account hash based on algorithm and public key or public_key_path

        :param algorithm: Algorithm used for generating keys, defaults to ed25519. See consts.SUPPORTED_KEY_ALGORITHMS
        :param public_key:          Public Key as bytes
        :param public_key_pem_path: File path to public_key pem file
        """
        key_holder = key_holders.key_holder_object(
            algorithm=algorithm,
            public_key=public_key,
            public_key_pem_path=public_key_pem_path,
        )
        return key_holder.account_hash



# -*- coding: utf-8 -*-
from typing import Dict

from casperlabs_client.decorators import guarded_command
from casperlabs_client import io, CasperLabsClient

NAME: str = "show-deploys"
HELP: str = "View deploys included in a block."
OPTIONS = [[("hash",), dict(type=str, help="Value of the block hash, base16 encoded.")]]


@guarded_command
def method(casperlabs_client: CasperLabsClient, args: Dict):
    response = casperlabs_client.show_deploys(args.get("hash"), full_view=False)
    io.print_blocks(response, element_name="deploy")
